@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2025.07.22 22:00:00                  #
+# Updated Date: 2025.08.01 03:00:00                  #
 # ================================================== #
 
 from typing import Any
@@ -38,12 +38,21 @@ class Llama:
                 "value": 75,
                 "multiplier": 1,
             },
+            "agent.llama.loop.mode": {
+                "type": "combo",
+                "label": "agent.llama.loop.mode",
+                "keys": [
+                    {"complete": trans("toolbox.agent.llama.loop.mode.complete")},
+                    {"score": trans("toolbox.agent.llama.loop.mode.score")},
+                ],
+            },
         }
 
     def setup(self):
         """Setup agent controller"""
         # register hooks
         self.window.ui.add_hook("update.global.agent.llama.loop.score", self.hook_update)
+        self.window.ui.add_hook("update.global.agent.llama.loop.mode", self.hook_update)
         self.reload()  # restore config
 
     def reload(self):
@@ -61,6 +70,13 @@ class Llama:
             option=self.options["agent.llama.loop.score"],
             value=self.window.core.config.get('agent.llama.loop.score'),
         )
+        # loop score mode
+        self.window.controller.config.apply_value(
+            parent_id="global",
+            key="agent.llama.loop.mode",
+            option=self.options["agent.llama.loop.mode"],
+            value=self.window.core.config.get('agent.llama.loop.mode'),
+        )
 
     def reset_eval_step(self):
         """Reset evaluation step"""
@@ -99,7 +115,7 @@ class Llama:
 
     def on_finish(self, ctx: CtxItem):
         """
-        Finish agent run
+        On finished agent run
 
         :param ctx: CtxItem
         """
@@ -107,6 +123,11 @@ class Llama:
             self.on_end()
             return  # abort if loop is disabled
 
+        # disable if bot
+        if self.window.controller.presets.is_bot():
+            self.on_end()
+            return  # abort if bot
+
         # check if not stopped
         if self.window.controller.kernel.stopped():
             self.on_end()
@@ -154,4 +175,8 @@ class Llama:
         if key == 'agent.llama.loop.score':
             self.window.core.config.set(key, int(value))
             self.window.core.config.save()
+            self.update()
+        elif key == 'agent.llama.loop.mode':
+            self.window.core.config.set(key, value)
+            self.window.core.config.save()
             self.update()
\ No newline at end of file
