@@ -6,11 +6,10 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.12.14 08:00:00                  #
+# Updated Date: 2025.08.03 14:00:00                  #
 # ================================================== #
 
 from typing import Optional
-import webbrowser
 
 from pygpt_net.item.assistant import AssistantItem
 
@@ -294,7 +293,7 @@ class Assistant:
 
     def goto_online(self):
         """Open Assistants page"""
-        webbrowser.open('https://platform.openai.com/assistants')
+        self.window.controller.dialogs.info.open_url('https://platform.openai.com/assistants')
 
     def change_locked(self) -> bool:
         """
