@@ -6,12 +6,13 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.12.14 08:00:00                  #
+# Updated Date: 2025.08.03 14:00:00                  #
 # ================================================== #
 
 from typing import Optional, Any
 
-from PySide6.QtWidgets import QApplication, QFileDialog
+from qasync import QApplication
+from PySide6.QtWidgets import QFileDialog
 
 from pygpt_net.utils import trans
 
