@@ -6,14 +6,14 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.12.14 08:00:00                  #
+# Updated Date: 2025.08.03 14:00:00                  #
 # ================================================== #
 
 import copy
 import json
 from typing import Optional
 
-from PySide6.QtWidgets import QApplication
+from qasync import QApplication
 
 from pygpt_net.item.assistant import AssistantStoreItem
 from pygpt_net.utils import trans
