@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2025.08.11 19:00:00                  #
+# Updated Date: 2025.08.14 01:00:00                  #
 # ================================================== #
 
 from typing import Dict, Any
@@ -164,14 +164,16 @@ class Response:
         :param context: BridgeContext
         :param extra: Extra data
         """
+        global_mode = self.window.core.config.get('mode', MODE_AGENT_LLAMA)
         ctx = context.ctx
         if self.window.controller.kernel.stopped():
             output = ctx.output
             if output and has_unclosed_code_tag(output):
                 ctx.output += "\n```"
             ctx.msg_id = None
-            self.window.core.ctx.add(ctx)  # store context to prevent current output from being lost
-            self.window.controller.ctx.prepare_name(ctx)  # summarize if not yet
+            if ctx.id is None:
+                self.window.core.ctx.add(ctx)  # store context to prevent current output from being lost
+                self.window.controller.ctx.prepare_name(ctx)  # summarize if not yet
 
             # finish render
             self.window.dispatch(AppEvent(AppEvent.CTX_END))  # app event
@@ -227,7 +229,8 @@ class Response:
         }
         event = RenderEvent(RenderEvent.INPUT_APPEND, data)
         self.window.dispatch(event)
-        self.window.core.ctx.add(ctx)
+        if ctx.id is None:
+            self.window.core.ctx.add(ctx)
         self.window.controller.ctx.update(
             reload=True,
             all=False,
@@ -266,6 +269,9 @@ class Response:
         self.window.dispatch(event)
 
         # if continue reasoning
+        if global_mode not in [MODE_AGENT_LLAMA, MODE_AGENT_OPENAI]:
+            return  # no agent mode, nothing to do
+
         if ctx.extra is None or (type(ctx.extra) == dict and "agent_finish" not in ctx.extra):
             self.window.update_status(trans("status.agent.reasoning"))
             self.window.controller.chat.common.lock_input()  # lock input, re-enable stop button
