@@ -6,12 +6,12 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2025.07.26 18:00:00                  #
+# Updated Date: 2025.08.03 14:00:00                  #
 # ================================================== #
 
 from typing import Any
 
-from PySide6.QtWidgets import QApplication
+from qasync import QApplication
 
 from pygpt_net.core.events import KernelEvent
 from pygpt_net.core.bridge.context import BridgeContext
@@ -61,6 +61,9 @@ class Stack:
 
         :param context: ReplyContext
         """
+        if context is None:
+            return
+
         # expert call
         if context.type == ReplyContext.EXPERT_CALL:
             self.window.core.experts.call(
