@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.12.14 08:00:00                  #
+# Updated Date: 2025.08.11 00:00:00                  #
 # ================================================== #
 
 from typing import List
@@ -38,11 +38,12 @@ class Bag:
 
         :param items: ctx items
         """
+        self.clear_items()
         self.items = items
 
     def clear_items(self):
         """Clear items"""
-        self.items = []
+        self.items.clear()
 
     def count_items(self) -> int:
         """
