@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2025.07.22 22:00:00                  #
+# Updated Date: 2025.08.14 01:00:00                  #
 # ================================================== #
 
 from typing import Any
@@ -28,7 +28,10 @@ class Response:
             model: ModelItem,
             response: Any
     ) -> None:
-        pass
+        output = str(response)
+        if output is None:
+            output = ""
+        ctx.set_output(output, "")
 
     def from_index(
             self,
