@@ -6,10 +6,12 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.11.26 04:00:00                  #
+# Updated Date: 2025.08.12 19:00:00                  #
 # ================================================== #
 
-class PidData():
+import io
+
+class PidData:
 
     def __init__(self, pid, meta=None):
         """Pid Data"""
@@ -18,5 +20,19 @@ class PidData():
         self.images_appended = []
         self.urls_appended = []
         self.files_appended = []
-        self.buffer = ""
+        self._buffer = io.StringIO()
         self.is_cmd = False
+
+    @property
+    def buffer(self) -> str:
+        return self._buffer.getvalue()
+
+    @buffer.setter
+    def buffer(self, value: str):
+        self._buffer.seek(0)
+        self._buffer.truncate(0)
+        if value:
+            self._buffer.write(value)
+
+    def append_buffer(self, text: str):
+        self._buffer.write(text)
\ No newline at end of file
