@@ -3,11 +3,10 @@
 # ================================================== #
 # This file is a part of PYGPT package               #
 # Website: https://pygpt.net                         #
-# GitHub:  https://github.com/szczyglis-dev/py-gpt    #
+# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2025.06.21 16:00:00                  #
-# ================================================== #
+# Updated Date: 2025.08.11 19:00:00                  #
 
 from datetime import datetime
 from typing import Optional, List
@@ -43,6 +42,15 @@ class Renderer(BaseRenderer):
         """
         self.pids = {}
 
+    def fresh(self, meta: CtxMeta):
+        """
+        Fresh renderer
+
+        :param meta: context PID
+        """
+        self.reset(meta)
+        self.clear_output(meta)
+
     def get_pid(self, meta: CtxMeta):
         """
         Get PID for context meta
@@ -333,7 +341,7 @@ class Renderer(BaseRenderer):
             self.append_block(meta)
             self.append_chunk_start(meta, item)
 
-        self.pids[pid].buffer += raw_chunk
+        self.pids[pid].append_buffer(raw_chunk)
         self.append(meta, item, self.helpers.format_chunk(text_chunk), "")
 
     def append_block(
@@ -528,9 +536,19 @@ class Renderer(BaseRenderer):
     def clear_all(self):
         """Clear all"""
         for node in self.get_all_nodes():
-            node.clear()
-            node.document().setDefaultStyleSheet("")
-            node.setStyleSheet("")
-            node.document().setMarkdown("")
-            node.document().setHtml("")
-            node.setPlainText("")
\ No newline at end of file
+            try:
+                node.clear()
+                node.document().setDefaultStyleSheet("")
+                node.setStyleSheet("")
+                node.document().setMarkdown("")
+                node.document().setHtml("")
+                node.setPlainText("")
+            except Exception as e:
+                pass
+
+    def remove_pid(self, pid: int):
+        """
+        Remove PID from renderer
+        """
+        if pid in self.pids:
+            del self.pids[pid]
\ No newline at end of file
