@@ -6,12 +6,12 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2025.07.20 23:00:00                  #
+# Updated Date: 2025.08.07 18:00:00                  #
 # ================================================== #
 
 import uuid
 from datetime import datetime
-from typing import Optional, Any, Dict, Tuple
+from typing import Optional, Any, Dict, Tuple, Union
 
 from PySide6.QtGui import QIcon
 from PySide6.QtWidgets import QVBoxLayout, QWidget, QLayout
@@ -277,6 +277,8 @@ class Tabs:
         tab = self.get_tab_by_pid(pid)
         if tab is None:
             return
+
+        tab.cleanup() # unload assigned data from memory
         column_idx = tab.column_idx
         self.window.ui.layout.get_tabs_by_idx(column_idx).removeTab(tab.idx)
         del self.pids[pid]
@@ -433,7 +435,7 @@ class Tabs:
         :return: Min index
         """
         min = 999999
-        column_idx = None
+        found_column_idx = None
         exists = False
         for pid in self.pids:
             tab = self.pids[pid]
@@ -442,8 +444,100 @@ class Tabs:
                     and tab.idx < min):
                 min = tab.idx
                 exists = True
-                column_idx = tab.column_idx
-        return min, column_idx, exists
+                found_column_idx = tab.column_idx
+        return min, found_column_idx, exists
+
+    def get_closest_idx_by_type_exists(
+            self,
+            current: Tab,
+            type: int,
+            column_idx: int = 0
+    ) -> Tuple[int, Union[int, None], bool]:
+        """
+        Get the closest index by type
+
+        :param current: Current tab
+        :param type: Tab type
+        :param column_idx: Column index
+        :return: Min index
+        """
+        if current is None:
+            return -1, None, False
+        curr_idx = current.idx
+        idx = -1
+        found_column_idx = None
+        exists = False
+        for pid in self.pids:
+            tab = self.pids[pid]
+            if (tab.type == type
+                    and tab.column_idx == column_idx
+                    and tab.idx < curr_idx
+                    and tab.idx > idx):
+                idx = tab.idx
+                exists = True
+                found_column_idx = tab.column_idx
+
+        return idx, found_column_idx, exists
+
+    def get_prev_idx_from(self, idx: int, column_idx: int = 0) -> Tuple[int, bool]:
+        """
+        Get previous index from the specified index
+
+        :param idx: Tab index
+        :param column_idx: Column index
+        :return: Previous index and exists flag
+        """
+        tabs = self.window.ui.layout.get_tabs_by_idx(column_idx)
+        if idx < 1:
+            return -1, False
+        prev_idx = idx - 1
+        if prev_idx < 0 or prev_idx >= tabs.count():
+            return -1, False
+        tab = self.get_tab_by_index(prev_idx, column_idx)
+        if tab is None:
+            return -1, False
+        if tab.column_idx != column_idx:
+            return -1, False
+        return prev_idx, True
+
+    def get_next_idx_from(self, idx: int, column_idx: int = 0) -> Tuple[int, bool]:
+        """
+        Get next index from the specified index
+
+        :param idx: Tab index
+        :param column_idx: Column index
+        :return: Next index and exists flag
+        """
+        tabs = self.window.ui.layout.get_tabs_by_idx(column_idx)
+        if idx < 0 or idx >= tabs.count():
+            return -1, False
+        next_idx = idx + 1
+        if next_idx < 0 or next_idx >= tabs.count():
+            return -1, False
+        tab = self.get_tab_by_index(next_idx, column_idx)
+        if tab is None:
+            return -1, False
+        if tab.column_idx != column_idx:
+            return -1, False
+        return next_idx, True
+
+    def get_max_idx_by_column(self, column_idx: int = 0) -> int:
+        """
+        Get max index in column
+
+        :param column_idx: Column index
+        :return: Max index
+        """
+        tabs = self.window.ui.layout.get_tabs_by_idx(column_idx)
+        if tabs.count() == 0:
+            return -1
+        max_idx = -1
+        for idx in range(tabs.count()):
+            tab = self.get_tab_by_index(idx, column_idx)
+            if tab is not None and tab.column_idx == column_idx:
+                if tab.idx > max_idx:
+                    max_idx = tab.idx
+        return max_idx
 
     def update(self):
         """Update tabs data (pids) from UI (all columns)"""
@@ -598,7 +692,10 @@ class Tabs:
         old_tabs.removeTab(tab.idx)
         new_column = self.window.ui.layout.get_column_by_idx(column_idx)
         new_tabs = new_column.get_tabs()
-        tab.idx = new_tabs.addTab(tab.child, QIcon(tab.icon), tab.title)
+        icon = QIcon()  # for test purposes only
+        if isinstance(tab.icon, str):
+            icon = QIcon(tab.icon)
+        tab.idx = new_tabs.addTab(tab.child, icon, tab.title)
         tab.parent = new_column
         tab.column_idx = column_idx
         self.update()
@@ -871,6 +968,7 @@ class Tabs:
         tab_widget = TabBody(self.window)
         tab_widget.append(widget)
         tab_widget.setLayout(layout)
+        tab_widget.add_ref(widget)  # add reference to widget
         return tab_widget
 
     def from_layout(self, layout: QLayout) -> TabBody:
