@@ -6,10 +6,14 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2025.07.13 01:00:00                  #
+# Updated Date: 2025.07.30 00:00:00                  #
 # ================================================== #
 
-AGENT_MODE_STEP = "agent.mode.step"
-AGENT_MODE_PLAN = "agent.mode.plan"
-AGENT_MODE_ASSISTANT = "agent.mode.assistant"
-AGENT_MODE_WORKFLOW = "agent.mode.workflow"
+AGENT_MODE_STEP = "agent.mode.step"  # LlamaIndex
+AGENT_MODE_PLAN = "agent.mode.plan"  # LlamaIndex
+AGENT_MODE_ASSISTANT = "agent.mode.assistant"  # LlamaIndex
+AGENT_MODE_WORKFLOW = "agent.mode.workflow"  # LlamaIndex
+AGENT_MODE_OPENAI = "agent.mode.openai"  # openai-agent
+
+AGENT_TYPE_LLAMA = "agent.type.llama"  # LlamaIndex
+AGENT_TYPE_OPENAI = "agent.type.openai"  # OpenAI Agent
