@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2024.11.18 21:00:00                  #
+# Updated Date: 2025.08.11 14:00:00                  #
 # ================================================== #
 
 from PySide6.QtCore import Slot
@@ -30,35 +30,39 @@ class Worker(BaseWorker):
 
     @Slot()
     def run(self):
-        responses = []
-        msg = None
-        for item in self.cmds:
-            if self.is_stopped():
-                break
-            response = None
-            try:
-                if item["cmd"] == "get_context":
-                    response = self.cmd_get_context(item)
+        try:
+            responses = []
+            msg = None
+            for item in self.cmds:
+                if self.is_stopped():
+                    break
+                response = None
+                try:
+                    if item["cmd"] == "get_context":
+                        response = self.cmd_get_context(item)
 
-                if response:
-                    responses.append(response)
+                    if response:
+                        responses.append(response)
 
-            except Exception as e:
-                msg = "Error: {}".format(e)
-                responses.append(
-                    self.make_response(
-                        item,
-                        self.throw_error(e)
+                except Exception as e:
+                    msg = "Error: {}".format(e)
+                    responses.append(
+                        self.make_response(
+                            item,
+                            self.throw_error(e)
+                        )
                     )
-                )
 
-        # send response
-        if len(responses) > 0:
-            self.reply_more(responses)
+            if len(responses) > 0:
+                self.reply_more(responses) # send response
 
-        # update status
-        if msg is not None:
-            self.status(msg)
+            if msg is not None:
+                self.status(msg) # update status
+
+        except Exception as e:
+            self.error(e)
+        finally:
+            self.cleanup()
 
     def cmd_get_context(self, item: dict) -> dict:
         """
