@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2025.08.02 20:00:00                  #
+# Updated Date: 2025.08.08 19:00:00                  #
 # ================================================== #
 
 from packaging.version import parse as parse_version, Version
@@ -710,6 +710,49 @@ class Patch:
                             model.mode.remove("agent_openai")
                 updated = True
 
+            # <  2.5.91  <--- GPT-5
+            if old < parse_version("2.5.91"):
+                print("Migrating models from < 2.5.91...")
+                if "gpt-5" not in data:
+                    data["gpt-5"] = base_data["gpt-5"]
+                if "gpt-5-mini" not in data:
+                    data["gpt-5-mini"] = base_data["gpt-5-mini"]
+                if "gpt-5-nano" not in data:
+                    data["gpt-5-nano"] = base_data["gpt-5-nano"]
+                updated = True
+
+            # <  2.5.93  <--- GPT-5 low and high
+            if old < parse_version("2.5.93"):
+                print("Migrating models from < 2.5.93...")
+                if "gpt-5-low" not in data:
+                    data["gpt-5-low"] = base_data["gpt-5-low"]
+                if "gpt-5-mini-low" not in data:
+                    data["gpt-5-mini-low"] = base_data["gpt-5-mini-low"]
+                if "gpt-5-nano-low" not in data:
+                    data["gpt-5-nano-low"] = base_data["gpt-5-nano-low"]
+                if "gpt-5-high" not in data:
+                    data["gpt-5-high"] = base_data["gpt-5-high"]
+                if "gpt-5-mini-high" not in data:
+                    data["gpt-5-mini-high"] = base_data["gpt-5-mini-high"]
+                if "gpt-5-nano-high" not in data:
+                    data["gpt-5-nano-high"] = base_data["gpt-5-nano-high"]
+                updated = True
+
+            # <  2.5.94  <--- gpt-oss
+            if old < parse_version("2.5.94"):
+                print("Migrating models from < 2.5.94...")
+                if "gpt-oss-20b" not in data:
+                    data["gpt-oss-20b"] = base_data["gpt-oss-20b"]
+                if "gpt-oss-120b" not in data:
+                    data["gpt-oss-120b"] = base_data["gpt-oss-120b"]
+                if "gpt-oss-20b-huggingface-router" not in data:
+                    data["gpt-oss-20b-huggingface-router"] = base_data["gpt-oss-20b-huggingface-router"]
+                if "gpt-oss-120b-huggingface-router" not in data:
+                    data["gpt-oss-120b-huggingface-router"] = base_data["gpt-oss-120b-huggingface-router"]
+                if "gpt-4.1-nano" not in data:
+                    data["gpt-4.1-nano"] = base_data["gpt-4.1-nano"]
+                updated = True
+
         # update file
         if updated:
             data = dict(sorted(data.items()))
