@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2025.07.25 18:00:00                  #
+# Updated Date: 2025.08.02 20:00:00                  #
 # ================================================== #
 
 import json
@@ -16,7 +16,7 @@ from typing import Optional, Dict, Any, List
 from pygpt_net.core.types import (
     MODE_CHAT,
     MODE_VISION,
-    MODE_AUDIO, MULTIMODAL_IMAGE, MODE_RESEARCH, MODE_COMPLETION,
+    MODE_AUDIO, MULTIMODAL_IMAGE, MODE_RESEARCH, MODE_COMPLETION, OPENAI_DISABLE_TOOLS,
 )
 from pygpt_net.core.bridge.context import BridgeContext, MultimodalContext
 from pygpt_net.item.ctx import CtxItem
@@ -84,7 +84,7 @@ class Chat:
             model.id,
         )
         # check if max tokens not exceeded
-        if max_tokens > 0:
+        if max_tokens > 0 and int(model.ctx) > 0:
             if msg_tokens + int(max_tokens) > model.ctx:
                 max_tokens = model.ctx - msg_tokens - 1
                 if max_tokens < 0:
@@ -111,12 +111,16 @@ class Chat:
         if model.extra and "reasoning_effort" in model.extra:
             response_kwargs['reasoning_effort'] = model.extra["reasoning_effort"]
 
+        # tool calls are not supported for some models
+        if model.id in OPENAI_DISABLE_TOOLS:
+            tools = []  # force disable
+
         # append tools
         if len(tools) > 0:
             response_kwargs['tools'] = tools
 
         if max_tokens > 0:
-            if not model.id.startswith("o1") and not model.id.startswith("o3"):
+            if model.id is None or (not model.id.startswith("o1") and not model.id.startswith("o3")):
                 response_kwargs['max_tokens'] = max_tokens
             else:
                 response_kwargs['max_completion_tokens'] = max_tokens
@@ -184,7 +188,7 @@ class Chat:
         max_ctx_tokens = self.window.core.config.get('max_total_tokens')  # max context window
 
         # fit to max model tokens
-        if max_ctx_tokens > model.ctx:
+        if max_ctx_tokens > model.ctx and model.ctx > 0:
             max_ctx_tokens = model.ctx
 
         # input tokens: reset
