@@ -6,7 +6,7 @@
 # GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2025.08.09 19:00:00                  #
+# Updated Date: 2025.08.14 13:00:00                  #
 # ================================================== #
 
 from PySide6.QtCore import Qt
@@ -72,18 +72,17 @@ class Preset(BaseConfigDialog):
         fields = self.window.controller.presets.editor.get_options()
 
         # build settings widgets
-        widgets = self.build_widgets(self.id, fields)  # from base config dialog
+        widgets = self.build_widgets(self.id, fields, excluded=["tool.function"])  # from base config dialog
 
         # apply settings widgets
         for key in widgets:
             self.window.ui.config[self.id][key] = widgets[key]
 
-        # btn: add function
-        self.window.ui.config[self.id]['tool.function'].add_btn.setText(trans('assistant.func.add'))
-
         # apply widgets to layouts
         options = {}
         for key in widgets:
+            if key == "tool.function":
+                continue
             if fields[key]["type"] in ['text', 'int', 'float']:
                 # if key != "prompt":  # built separately
                 options[key] = self.add_option(widgets[key], fields[key])
@@ -97,7 +96,6 @@ class Preset(BaseConfigDialog):
                     widgets[key].setMinimumHeight(100)
                 options[key] = self.add_row_option(widgets[key], fields[key])
             elif fields[key]["type"] == 'bool':
-                # widgets[key].setMaximumHeight(38)
                 options[key] = self.add_raw_option(widgets[key], fields[key])
             elif fields[key]["type"] == 'dict':
                 options[key] = self.add_row_option(widgets[key], fields[key])
@@ -106,8 +104,6 @@ class Preset(BaseConfigDialog):
             elif fields[key]["type"] == 'bool_list':
                 options[key] = self.add_row_option(widgets[key], fields[key])
 
-        self.window.ui.nodes['preset.tool.function.label'].setVisible(False)  # hide label
-
         rows = QVBoxLayout()
 
         for key in options:
@@ -156,68 +152,31 @@ class Preset(BaseConfigDialog):
         rows_mode.addLayout(rows_mode_middle)
         rows_mode.addLayout(rows_mode_right)
         rows_mode.setAlignment(Qt.AlignTop)
+        rows_mode.setContentsMargins(0, 0, 0, 0)
         rows_mode.addStretch(1)
 
         # modes
         self.window.ui.nodes['preset.editor.modes'] = QWidget()
         self.window.ui.nodes['preset.editor.modes'].setLayout(rows_mode)
         self.window.ui.nodes['preset.editor.modes'].setContentsMargins(0, 0, 0, 0)
-       # self.window.ui.nodes['preset.editor.modes'].setMaximumWidth(300)
-
-        # functions label
-        self.window.ui.nodes['preset.tool.function.label.all'] = HelpLabel(
-            trans("preset.tool.function.tip.all"))
-        self.window.ui.nodes['preset.tool.function.label.all'].setAlignment(Qt.AlignCenter)
-        self.window.ui.nodes['preset.tool.function.label.assistant'] = HelpLabel(
-            trans("preset.tool.function.tip.assistant"))
-        self.window.ui.nodes['preset.tool.function.label.assistant'].setAlignment(Qt.AlignCenter)
-        self.window.ui.nodes['preset.tool.function.label.agent_llama'] = HelpLabel(
-            trans("preset.tool.function.tip.agent_llama"))
-        self.window.ui.nodes['preset.tool.function.label.agent_llama'].setAlignment(Qt.AlignCenter)
-
-        # functions
-        self.window.ui.nodes['preset.editor.functions'] = QWidget()
-        self.window.ui.nodes['preset.editor.functions'].setLayout(options["tool.function"])
 
         # experts
         self.window.ui.nodes['preset.editor.experts'] = ExpertsEditor(self.window)
 
-        # agents - llama index
-        agent_keys = [
-            "idx",
-            "assistant_id",
-        ]
-        agent_layout = QVBoxLayout()
-        agent_layout.setContentsMargins(0, 0, 0, 0)
-        for key in agent_keys:
-            widget = QWidget()
-            widget.setLayout(options[key])
-            agent_layout.addWidget(widget)
-        agent_layout.addStretch()
-        self.window.ui.nodes['preset.editor.agent_llama'] = QWidget()
-        self.window.ui.nodes['preset.editor.agent_llama'].setLayout(agent_layout)
-        self.window.ui.nodes['preset.editor.agent_llama'].setContentsMargins(20, 0, 0, 30)
-
         # desc and prompt
-
         self.window.ui.nodes['preset.editor.description'] = QWidget()
         self.window.ui.nodes['preset.editor.description'].setLayout(options['description'])
         self.window.ui.nodes['preset.editor.description'].setContentsMargins(0, 5, 0, 5)
-        ''''        
-        self.window.ui.nodes['preset.editor.remote_tools'] = QWidget()
-        self.window.ui.nodes['preset.editor.remote_tools'].setLayout(options['remote_tools'])
-        self.window.ui.nodes['preset.editor.remote_tools'].setContentsMargins(0, 0, 0, 0)
-        '''
 
         # prompt + extra options
         prompt_layout = QVBoxLayout()
         prompt_layout.addWidget(widgets['prompt'])
         prompt_layout.setContentsMargins(0, 10, 0, 10)
+
         footer_layout = self.prepare_extra_config(prompt_layout)
 
         prompt_layout = QVBoxLayout()
         prompt_layout.setContentsMargins(0, 0, 0, 0)
-        # prompt_layout.addWidget(self.window.ui.nodes['preset.editor.remote_tools'])
         prompt_layout.addWidget(self.window.ui.nodes['preset.editor.description'])
         prompt_layout.addLayout(footer_layout)
 
@@ -234,7 +193,7 @@ class Preset(BaseConfigDialog):
             "temperature",
             "agent_provider",
             "agent_provider_openai",
-            "remote_tools",
+            "idx",
         ]
         personalize_keys = [
             "ai_name",
@@ -248,6 +207,18 @@ class Preset(BaseConfigDialog):
             self.window.ui.nodes['preset.editor.' + key].setContentsMargins(0, 0, 0, 0)
             rows.addWidget(self.window.ui.nodes['preset.editor.' + key])
 
+        # remote tools
+        self.window.ui.nodes['preset.editor.remote_tools'] = QWidget()
+        self.window.ui.nodes['preset.editor.remote_tools'].setLayout(options['remote_tools'])
+        self.window.ui.nodes['preset.editor.remote_tools'].setContentsMargins(0, 0, 0, 0)
+
+        rows_remote_tools = QVBoxLayout()
+        rows_remote_tools.addWidget(self.window.ui.nodes['preset.editor.remote_tools'])
+        rows_remote_tools.addStretch(1)
+
+        widget_remote_tools = QWidget()
+        widget_remote_tools.setLayout(rows_remote_tools)
+
         # personalize
         personalize_rows = QVBoxLayout()
         for key in personalize_keys:
@@ -267,40 +238,18 @@ class Preset(BaseConfigDialog):
         personalize_rows.addStretch(1)
         personalize_rows.addWidget(warn_label)
 
-        self.window.ui.nodes['preset.editor.remote_tools'].setMinimumHeight(140)
-
         rows.setContentsMargins(0, 0, 0, 0)
         rows.addStretch(1)
         rows.setAlignment(Qt.AlignTop)
+
         widget_base = QWidget()
         widget_base.setLayout(rows)
         widget_base.setMinimumWidth(300)
 
-        func_tip_layout = QVBoxLayout()
-        func_tip_layout.addWidget(self.window.ui.nodes['preset.tool.function.label.all'])
-        func_tip_layout.addWidget(self.window.ui.nodes['preset.tool.function.label.assistant'])
-        func_tip_layout.addWidget(self.window.ui.nodes['preset.tool.function.label.agent_llama'])
-        func_tip_layout.setContentsMargins(0, 0, 0, 0)
-        func_tip_widget = QWidget()
-        func_tip_widget.setLayout(func_tip_layout)
-
-        func_rows = QVBoxLayout()
-        func_rows.addWidget(self.window.ui.nodes['preset.editor.functions'])
-        func_rows.addWidget(self.window.ui.nodes['preset.editor.experts'])
-        func_rows.addWidget(self.window.ui.nodes['preset.editor.agent_llama'])
-        #func_rows.addStretch()
-        func_rows.addWidget(func_tip_widget)
-
-        func_rows.setContentsMargins(0, 0, 0, 0)
-        func_widget = QWidget()
-        func_widget.setLayout(func_rows)
-        #func_widget.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
-        #self.window.ui.nodes['preset.editor.functions'].setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
         self.window.ui.nodes['preset.editor.experts'].setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
 
         main = QHBoxLayout()
         main.addWidget(widget_base)
-        main.addWidget(func_widget)
         main.addWidget(self.window.ui.nodes['preset.editor.modes'])
 
         widget_main = QWidget()
@@ -311,17 +260,24 @@ class Preset(BaseConfigDialog):
         self.window.ui.splitters['editor.presets'].addWidget(widget_prompt)
         self.window.ui.splitters['editor.presets'].setStretchFactor(0, 1)
         self.window.ui.splitters['editor.presets'].setStretchFactor(1, 2)
-        #self.window.ui.splitters['editor.presets'].setChildrenCollapsible(False)
 
         widget_personalize = QWidget()
         widget_personalize.setLayout(personalize_rows)
 
-        tabs = QTabWidget()
-        tabs.addTab(self.window.ui.splitters['editor.presets'], trans("preset.tab.general"))
-        tabs.addTab(widget_personalize, trans("preset.tab.personalize"))
+        experts_rows = QVBoxLayout()
+        experts_rows.addWidget(self.window.ui.nodes['preset.editor.experts'])
+
+        widget_experts = QWidget()
+        widget_experts.setLayout(experts_rows)
+
+        self.window.ui.tabs['preset.editor.tabs'] = QTabWidget()
+        self.window.ui.tabs['preset.editor.tabs'].addTab(self.window.ui.splitters['editor.presets'], trans("preset.tab.general"))
+        self.window.ui.tabs['preset.editor.tabs'].addTab(widget_personalize, trans("preset.tab.personalize"))
+        self.window.ui.tabs['preset.editor.tabs'].addTab(widget_experts, trans("preset.tab.experts"))
+        self.window.ui.tabs['preset.editor.tabs'].addTab(widget_remote_tools, trans("preset.tab.remote_tools"))
 
         layout = QVBoxLayout()
-        layout.addWidget(tabs)
+        layout.addWidget(self.window.ui.tabs['preset.editor.tabs'])
         layout.addLayout(footer)
 
         self.window.ui.dialog['editor.' + self.dialog_id] = EditorDialog(self.window, self.dialog_id)
@@ -343,8 +299,9 @@ class Preset(BaseConfigDialog):
         prompt_layout.setContentsMargins(0, 10, 0, 10)
         prompt_widget = QWidget()
         prompt_widget.setLayout(prompt_layout)
-        self.window.ui.tabs['preset.editor.extra'] = QTabWidget()
+
         self.window.ui.nodes['preset.editor.extra'] = {}
+        self.window.ui.tabs['preset.editor.extra'] = QTabWidget()
         self.window.ui.tabs['preset.editor.extra'].addTab(
             prompt_widget,
             trans("preset.prompt"),
