@@ -3,10 +3,10 @@
 # ================================================== #
 # This file is a part of PYGPT package               #
 # Website: https://pygpt.net                         #
-# GitHub:  https://github.com/szczyglis-dev/py-gpt    #
+# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
 # MIT License                                        #
 # Created By  : Marcin Szczygliński                  #
-# Updated Date: 2025.07.20 23:00:00                  #
+# Updated Date: 2025.08.16 00:00:00                  #
 # ================================================== #
 
 import datetime
@@ -35,34 +35,63 @@ class ContextList(BaseList):
         self.window = window
         self.id = id
         self.expanded_items = set()
-        self.setItemDelegate(ImportantItemDelegate())
+        self._icons = {
+            'add': QIcon(":/icons/add.svg"),
+            'edit': QIcon(":/icons/edit.svg"),
+            'delete': QIcon(":/icons/delete.svg"),
+            'chat': QIcon(":/icons/chat.svg"),
+            'copy': QIcon(":/icons/copy.svg"),
+            'close': QIcon(":/icons/close.svg"),
+            'pin': QIcon(":/icons/pin.svg"),
+            'clock': QIcon(":/icons/clock.svg"),
+            'db': QIcon(":/icons/db.svg"),
+            'folder': QIcon(":/icons/folder_filled.svg"),
+            'attachment': QIcon(":/icons/attachment.svg"),
+        }
+        self._color_icon_cache = {}
+        self.setItemDelegate(ImportantItemDelegate(self, self._icons['attachment']))
         self.setContextMenuPolicy(Qt.CustomContextMenu)
         self.customContextMenuRequested.connect(self.show_context_menu)
         self._backup_selection = None
         self.restore_after_ctx_menu = True
 
+    @property
+    def _model(self):
+        return self.window.ui.models['ctx.list']
+
+    @property
+    def _view(self):
+        return self.window.ui.nodes['ctx.list']
+
+    def _color_icon(self, color: QColor) -> QIcon:
+        key = color.rgba()
+        icon = self._color_icon_cache.get(key)
+        if icon is None:
+            pixmap = QPixmap(16, 16)
+            pixmap.fill(color)
+            icon = QIcon(pixmap)
+            self._color_icon_cache[key] = icon
+        return icon
+
     def click(self, index):
         """
         Click event (override, connected in BaseList class)
 
         :param index: index
         """
-        item = self.window.ui.models['ctx.list'].itemFromIndex(index)
-        if item is not None:
-            if not hasattr(item, 'isFolder'):
-                return
-            if item.isFolder:
-                self.window.controller.ctx.set_group(item.id)
-                if self.window.ui.nodes['ctx.list'].isExpanded(index):
-                    self.expanded_items.discard(item.id)
-                    self.window.ui.nodes['ctx.list'].collapse(index)
-                else:
-                    self.expanded_items.add(item.id)
-                    self.window.ui.nodes['ctx.list'].expand(index)
+        item = self._model.itemFromIndex(index)
+        if item is None or not hasattr(item, 'isFolder'):
+            return
+        if item.isFolder:
+            self.window.controller.ctx.set_group(item.id)
+            if self._view.isExpanded(index):
+                self.expanded_items.discard(item.id)
+                self._view.collapse(index)
             else:
-                self.window.controller.ctx.select_by_id(item.id)
+                self.expanded_items.add(item.id)
+                self._view.expand(index)
         else:
-            pass  # unselected
+            self.window.controller.ctx.select_by_id(item.id)
 
     def expand_group(self, id):
         """
@@ -70,13 +99,13 @@ class ContextList(BaseList):
 
         :param id: group id
         """
-        for i in range(self.window.ui.models['ctx.list'].rowCount()):
-            item = self.window.ui.models['ctx.list'].item(i)
-            if isinstance(item, GroupItem):
-                if item.id == id:
-                    index = self.window.ui.models['ctx.list'].indexFromItem(item)
-                    self.window.ui.nodes['ctx.list'].expand(index)
-                    self.expanded_items.add(id)
+        for i in range(self._model.rowCount()):
+            item = self._model.item(i)
+            if isinstance(item, GroupItem) and item.id == id:
+                index = self._model.indexFromItem(item)
+                self._view.expand(index)
+                self.expanded_items.add(id)
+                break
 
     def dblclick(self, index):
         """
@@ -96,11 +125,10 @@ class ContextList(BaseList):
         elif event.button() == Qt.RightButton:
             index = self.indexAt(event.pos())
             if index.isValid():
-                self._backup_selection = list(self.selectionModel().selectedIndexes())
-                self.selectionModel().clearSelection()
-                self.selectionModel().select(
-                    index, QItemSelectionModel.Select | QItemSelectionModel.Rows
-                )
+                sel = self.selectionModel()
+                self._backup_selection = list(sel.selectedIndexes())
+                sel.clearSelection()
+                sel.select(index, QItemSelectionModel.Select | QItemSelectionModel.Rows)
             event.accept()
         else:
             super().mousePressEvent(event)
@@ -113,45 +141,24 @@ class ContextList(BaseList):
         """
         global_pos = self.viewport().mapToGlobal(pos)
         index = self.indexAt(pos)
-        item = self.window.ui.models['ctx.list'].itemFromIndex(index)
-
-        if (item is not None
-                and index.isValid()
-                and hasattr(item, 'id')):
+        item = self._model.itemFromIndex(index)
 
+        if item is not None and index.isValid() and hasattr(item, 'id'):
             idx = item.row()
             id_value = item.id
 
-            # group context menu
             if hasattr(item, 'isFolder') and item.isFolder:
-                actions = {}
-                actions['new'] = QAction(QIcon(":/icons/add.svg"), trans('action.ctx.new'), self)
-                actions['new'].triggered.connect(
-                    functools.partial(self.window.controller.ctx.new, force=False, group_id=id_value)
-                )
-                actions['rename'] = QAction(QIcon(":/icons/edit.svg"), trans('action.rename'), self)
-                actions['rename'].triggered.connect(
-                    functools.partial(self.window.controller.ctx.rename_group, id_value)
-                )
-                actions['delete'] = QAction(QIcon(":/icons/delete.svg"), trans('action.group.delete.only'), self)
-                actions['delete'].triggered.connect(
-                    functools.partial(self.window.controller.ctx.delete_group, id_value)
-                )
-                actions['delete_all'] = QAction(QIcon(":/icons/delete.svg"), trans('action.group.delete.all'), self)
-                actions['delete_all'].triggered.connect(
-                    functools.partial(self.window.controller.ctx.delete_group_all, id_value)
-                )
-
                 menu = QMenu(self)
-                menu.addAction(actions['new'])
-                menu.addAction(actions['rename'])
-                menu.addAction(actions['delete'])
-                menu.addAction(actions['delete_all'])
-
+                a_new = menu.addAction(self._icons['add'], trans('action.ctx.new'))
+                a_new.triggered.connect(functools.partial(self.window.controller.ctx.new_in_group, force=False, group_id=id_value))
+                a_rename = menu.addAction(self._icons['edit'], trans('action.rename'))
+                a_rename.triggered.connect(functools.partial(self.window.controller.ctx.rename_group, id_value))
+                a_delete = menu.addAction(self._icons['delete'], trans('action.group.delete.only'))
+                a_delete.triggered.connect(functools.partial(self.window.controller.ctx.delete_group, id_value))
+                a_delete_all = menu.addAction(self._icons['delete'], trans('action.group.delete.all'))
+                a_delete_all.triggered.connect(functools.partial(self.window.controller.ctx.delete_group_all, id_value))
                 if idx >= 0:
-                    menu.exec_(global_pos)
-
-            # children context menu
+                    menu.exec(global_pos)
             else:
                 ctx_id = id_value
                 ctx = self.window.core.ctx.get_meta_by_id(ctx_id)
@@ -160,169 +167,111 @@ class ContextList(BaseList):
 
                 is_important = ctx.important
 
-                actions = {}
-                actions['open'] = QAction(QIcon(":/icons/chat.svg"), trans('action.open'), self)
-                actions['open'].triggered.connect(
-                    functools.partial(self.action_open, ctx_id, idx)
-                )
+                menu = QMenu(self)
+                a_open = menu.addAction(self._icons['chat'], trans('action.open'))
+                a_open.triggered.connect(functools.partial(self.action_open, ctx_id, idx))
 
-                actions['open_new_tab'] = QAction(QIcon(":/icons/chat.svg"), trans('action.open_new_tab'), self)
-                actions['open_new_tab'].triggered.connect(
-                    functools.partial(self.action_open_new_tab, ctx_id, idx)
-                )
+                a_open_new_tab = menu.addAction(self._icons['chat'], trans('action.open_new_tab'))
+                a_open_new_tab.triggered.connect(functools.partial(self.action_open_new_tab, ctx_id, idx))
 
-                actions['rename'] = QAction(QIcon(":/icons/edit.svg"), trans('action.rename'), self)
-                actions['rename'].triggered.connect(
-                    functools.partial(self.action_rename, ctx_id)
-                )
+                a_rename = menu.addAction(self._icons['edit'], trans('action.rename'))
+                a_rename.triggered.connect(functools.partial(self.action_rename, ctx_id))
+
+                a_duplicate = menu.addAction(self._icons['copy'], trans('action.duplicate'))
+                a_duplicate.triggered.connect(functools.partial(self.action_duplicate, ctx_id))
 
                 if is_important:
-                    actions['important'] = QAction(QIcon(":/icons/pin.svg"), trans('action.unpin'), self)
-                    actions['important'].triggered.connect(
-                        functools.partial(self.action_unpin, ctx_id)
-                    )
+                    a_pin = menu.addAction(self._icons['pin'], trans('action.unpin'))
+                    a_pin.triggered.connect(functools.partial(self.action_unpin, ctx_id))
                 else:
-                    actions['important'] = QAction(QIcon(":/icons/pin.svg"), trans('action.pin'), self)
-                    actions['important'].triggered.connect(
-                        functools.partial(self.action_pin, ctx_id)
-                    )
+                    a_pin = menu.addAction(self._icons['pin'], trans('action.pin'))
+                    a_pin.triggered.connect(functools.partial(self.action_pin, ctx_id))
 
-                actions['duplicate'] = QAction(QIcon(":/icons/copy.svg"), trans('action.duplicate'), self)
-                actions['duplicate'].triggered.connect(
-                    functools.partial(self.action_duplicate, ctx_id)
-                )
+                a_delete = menu.addAction(self._icons['delete'], trans('action.delete'))
+                a_delete.triggered.connect(functools.partial(self.action_delete, ctx_id))
 
-                actions['delete'] = QAction(QIcon(":/icons/delete.svg"), trans('action.delete'), self)
-                actions['delete'].triggered.connect(
-                    functools.partial(self.action_delete, ctx_id)
-                )
-
-                actions['copy_id'] = QAction(QIcon(":/icons/copy.svg"), trans('action.ctx_copy_id') + " @" + str(ctx_id), self)
-                actions['copy_id'].triggered.connect(
-                    functools.partial(self.action_copy_id, ctx_id)
-                )
-
-                actions['reset'] = QAction(QIcon(":/icons/close.svg"), trans('action.ctx_reset'), self)
-                actions['reset'].triggered.connect(
-                    functools.partial(self.action_reset, ctx_id)
-                )
-
-                menu = QMenu(self)
-                menu.addAction(actions['open'])
-                menu.addAction(actions['open_new_tab'])
-                menu.addAction(actions['rename'])
-                menu.addAction(actions['duplicate'])
-                menu.addAction(actions['important'])
-                menu.addAction(actions['delete'])
-
-                # set label menu
                 colors = self.window.controller.ui.get_colors()
                 set_label_menu = menu.addMenu(trans('calendar.day.label'))
                 for status_id, status_info in colors.items():
-                    name = trans('calendar.day.' + status_info['label'])
-                    if status_id == 0:
-                        name = '-'
-                    color = status_info['color']
-                    pixmap = QPixmap(16, 16)
-                    pixmap.fill(color)
-                    icon = QIcon(pixmap)
-                    status_action = QAction(icon, name, self)
+                    name = trans('calendar.day.' + status_info['label']) if status_id != 0 else '-'
+                    icon = self._color_icon(status_info['color'])
+                    status_action = set_label_menu.addAction(icon, name)
                     status_action.triggered.connect(
                         functools.partial(self.window.controller.ctx.set_label, ctx_id, status_id)
                     )
-                    set_label_menu.addAction(status_action)
 
                 idx_menu = QMenu(trans('action.idx'), self)
-
-                # indexes list
                 idxs = self.window.core.config.get('llama.idx.list')
-                store = self.window.core.idx.get_current_store()  # get current idx store provider
-                if len(idxs) > 0:
+                store = self.window.core.idx.get_current_store()
+                if idxs:
                     for idx_dict in idxs:
                         index_id = idx_dict['id']
                         name = idx_dict['name'] + " (" + idx_dict['id'] + ")"
-                        action = idx_menu.addAction("IDX: " + name)
-                        action.setIcon(QIcon(":/icons/db.svg"))
-                        action.triggered.connect(
-                            functools.partial(self.action_idx, ctx_id, index_id)
-                        )
+                        action = idx_menu.addAction(self._icons['db'], "IDX: " + name)
+                        action.triggered.connect(functools.partial(self.action_idx, ctx_id, index_id))
 
-                    # remove from index
                     if ctx.indexed is not None and ctx.indexed > 0:
                         if store in ctx.indexes:
                             store_indexes = ctx.indexes[store]
                             idx_menu.addSeparator()
                             for store_index in store_indexes:
-                                action = idx_menu.addAction(trans("action.idx.remove") + ": " + store_index)
-                                action.setIcon(QIcon(":/icons/delete.svg"))
+                                action = idx_menu.addAction(self._icons['delete'], trans("action.idx.remove") + ": " + store_index)
                                 action.triggered.connect(
                                     functools.partial(self.action_idx_remove, store_index, ctx_id)
                                 )
                     menu.addMenu(idx_menu)
 
-                # move to group menu
                 group_menu = QMenu(trans('action.move_to'), self)
                 groups = self.window.core.ctx.get_groups()
 
-                # add group
-                action = group_menu.addAction(trans("action.group.new"))
-                action.setIcon(QIcon(":/icons/add.svg"))
-                action.triggered.connect(
-                    functools.partial(self.window.controller.ctx.new_group, ctx_id)
-                )
+                action = group_menu.addAction(self._icons['add'], trans("action.group.new"))
+                action.triggered.connect(functools.partial(self.window.controller.ctx.new_group, ctx_id))
 
-                if len(groups) > 0:
+                if groups:
                     group_menu.addSeparator()
 
                 for group_id, group in groups.items():
-                    action = group_menu.addAction(group.name)
-                    action.setIcon(QIcon(":/icons/folder_filled.svg"))
-                    action.triggered.connect(
-                        functools.partial(self.window.controller.ctx.move_to_group, ctx_id, group_id)
-                    )
+                    action = group_menu.addAction(self._icons['folder'], group.name)
+                    action.triggered.connect(functools.partial(self.window.controller.ctx.move_to_group, ctx_id, group_id))
 
-                if len(groups) > 0:
+                if groups:
                     group_menu.addSeparator()
 
                 if ctx.group_id is not None and ctx.group_id > 0:
                     group_name = str(ctx.group_id)
                     if ctx.group_id in groups:
                         group_name = groups[ctx.group_id].name
-                    action = group_menu.addAction(trans("action.group.remove") + ": " + group_name)
-                    action.setIcon(QIcon(":/icons/delete.svg"))
-                    action.triggered.connect(
-                        functools.partial(self.window.controller.ctx.remove_from_group, ctx_id)
-                    )
+                    action = group_menu.addAction(self._icons['delete'], trans("action.group.remove") + ": " + group_name)
+                    action.triggered.connect(functools.partial(self.window.controller.ctx.remove_from_group, ctx_id))
 
                 menu.addMenu(group_menu)
 
-                menu.addAction(actions['copy_id'])
+                a_copy_id = menu.addAction(self._icons['copy'], trans('action.ctx_copy_id') + " @" + str(ctx_id))
+                a_copy_id.triggered.connect(functools.partial(self.action_copy_id, ctx_id))
 
-                # show last indexed date if available
                 if ctx.indexed is not None and ctx.indexed > 0:
                     suffix = ""
                     if ctx.updated > ctx.indexed:
                         suffix = " *"
                     dt = datetime.datetime.fromtimestamp(ctx.indexed).strftime("%Y-%m-%d %H:%M")
-                    action = QAction(QIcon(":/icons/clock.svg"), trans('action.ctx.indexed') + ": " + dt + suffix, self)
+                    action = menu.addAction(self._icons['clock'], trans('action.ctx.indexed') + ": " + dt + suffix)
                     action.setEnabled(False)
-                    menu.addAction(action)
 
-                menu.addAction(actions['reset'])
+                a_reset = menu.addAction(self._icons['close'], trans('action.ctx_reset'))
+                a_reset.triggered.connect(functools.partial(self.action_reset, ctx_id))
 
                 if idx >= 0:
                     self.window.controller.ctx.set_selected(ctx_id)
-                    menu.exec_(global_pos)
+                    menu.exec(global_pos)
 
         self.store_scroll_position()
 
         if self.restore_after_ctx_menu:
             if self._backup_selection is not None:
-                self.selectionModel().clearSelection()
+                sel = self.selectionModel()
+                sel.clearSelection()
                 for sel_idx in self._backup_selection:
-                    self.selectionModel().select(
-                        sel_idx, QItemSelectionModel.Select | QItemSelectionModel.Rows
-                    )
+                    sel.select(sel_idx, QItemSelectionModel.Select | QItemSelectionModel.Rows)
                 self._backup_selection = None
 
         self.restore_after_ctx_menu = True
@@ -451,14 +400,29 @@ class ImportantItemDelegate(QtWidgets.QStyledItemDelegate):
     """
     Label color delegate
     """
+    def __init__(self, parent=None, attachment_icon: QIcon = None):
+        super().__init__(parent)
+        self._attachment_icon = attachment_icon or QIcon(":/icons/attachment.svg")
+        self._status_colors = {
+            0: QColor(100, 100, 100),
+            1: QColor(255, 0, 0),
+            2: QColor(255, 165, 0),
+            3: QColor(255, 255, 0),
+            4: QColor(0, 255, 0),
+            5: QColor(0, 0, 255),
+            6: QColor(75, 0, 130),
+            7: QColor(238, 130, 238),
+        }
+        self._pin_pen = QtGui.QPen(QtCore.Qt.black, 0.5, QtCore.Qt.SolidLine)
+
     def paint(self, painter, option, index):
         if index.parent().isValid():
             option.rect.adjust(15, 0, 0, 0)
 
         super(ImportantItemDelegate, self).paint(painter, option, index)
 
-        if index.data(QtCore.Qt.ItemDataRole.UserRole):
-            data = index.data(QtCore.Qt.ItemDataRole.UserRole)
+        data = index.data(QtCore.Qt.ItemDataRole.UserRole)
+        if data:
             label = data.get("label", 0)
             is_important = data.get("is_important", False)
             is_attachment = data.get("is_attachment", False)
@@ -466,44 +430,37 @@ class ImportantItemDelegate(QtWidgets.QStyledItemDelegate):
             painter.save()
 
             if is_attachment:
-                icon = QtGui.QIcon(":/icons/attachment.svg")
                 icon_size = option.decorationSize or QtCore.QSize(16, 16)
                 icon_pos = option.rect.right() - icon_size.width()
+                y = option.rect.top() + (option.rect.height() - icon_size.height()) // 2
                 icon_rect = QtCore.QRect(
                     icon_pos,
-                    option.rect.top() + (option.rect.height() - icon_size.height()) / 2,
+                    y,
                     icon_size.width(),
                     icon_size.height()
                 )
-                icon.paint(painter, icon_rect, QtCore.Qt.AlignCenter)
+                self._attachment_icon.paint(painter, icon_rect, QtCore.Qt.AlignCenter)
 
             if is_important:
                 color = self.get_color_for_status(3)
                 square_size = 3
-                square_margin = 0
                 square_rect = QtCore.QRect(
-                    option.rect.left() + square_margin,
+                    option.rect.left(),
                     option.rect.top() + 2,
                     square_size,
                     square_size,
                 )
                 painter.setBrush(color)
-                painter.setPen(
-                    QtGui.QPen(
-                        QtCore.Qt.black,
-                        0.5,
-                        QtCore.Qt.SolidLine,
-                    )
-                )
+                painter.setPen(self._pin_pen)
                 painter.drawRect(square_rect)
 
             if label > 0:
                 color = self.get_color_for_status(label)
                 square_size = 5
-                square_margin = 0
+                y = option.rect.center().y() - (square_size // 2) + 2
                 square_rect = QtCore.QRect(
-                    option.rect.left() + square_margin,
-                    option.rect.center().y() - (square_size / 2) + 2,
+                    option.rect.left(),
+                    y,
                     square_size,
                     square_size,
                 )
@@ -514,17 +471,7 @@ class ImportantItemDelegate(QtWidgets.QStyledItemDelegate):
             painter.restore()
 
     def get_color_for_status(self, status: int) -> QColor:
-        statuses = {
-            0: {'label': 'label.color.default', 'color': QColor(100, 100, 100)},
-            1: {'label': 'label.color.red', 'color': QColor(255, 0, 0)},
-            2: {'label': 'label.color.orange', 'color': QColor(255, 165, 0)},
-            3: {'label': 'label.color.yellow', 'color': QColor(255, 255, 0)},
-            4: {'label': 'label.color.green', 'color': QColor(0, 255, 0)},
-            5: {'label': 'label.color.blue', 'color': QColor(0, 0, 255)},
-            6: {'label': 'label.color.indigo', 'color': QColor(75, 0, 130)},
-            7: {'label': 'label.color.violet', 'color': QColor(238, 130, 238)},
-        }
-        return statuses.get(status, statuses[0])['color']
+        return self._status_colors.get(status, self._status_colors[0])
 
 
 class GroupItem(QStandardItem):
